@@ -40,7 +40,7 @@ from django.utils.six import text_type
 from packages.models import Package, PackageName, PackageString
 from arch.models import PackageArchitecture
 from util import get_url, download_url, response_is_valid, extract, \
-    get_sha1, get_sha256
+    get_checksum, Checksum
 from patchman.signals import progress_info_s, progress_update_s, \
     info_message, warning_message, error_message, debug_message
 
@@ -154,19 +154,6 @@ def get_primary_url(mirror_url, data):
     return primary_url, checksum, csum_type
 
 
-def get_sha(checksum_type, data):
-    """ Returns the checksum of the data. Returns None otherwise.
-    """
-    if checksum_type == 'sha' or checksum_type == 'sha1':
-        sha = get_sha1(data)
-    elif checksum_type == 'sha256':
-        sha = get_sha256(data)
-    else:
-        text = 'Unknown checksum type: {0!s}'.format(checksum_type)
-        error_message.send(sender=None, text=text)
-    return sha
-
-
 def find_mirror_url(stored_mirror_url, formats):
     """ Find the actual URL of the mirror by trying predefined paths
     """
@@ -227,26 +214,26 @@ def get_mirrorlist_urls(url):
                 return mirror_urls
 
 
-def add_mirrors_from_urls(mirror, mirror_urls):
+def add_mirrors_from_urls(repo, mirror_urls):
     """ Creates mirrors from a list of mirror urls
     """
     for mirror_url in mirror_urls:
         mirror_url = mirror_url.decode('ascii')
-        mirror_url = mirror_url.replace('$ARCH', mirror.repo.arch.name)
-        mirror_url = mirror_url.replace('$basearch', mirror.repo.arch.name)
+        mirror_url = mirror_url.replace('$ARCH', repo.arch.name)
+        mirror_url = mirror_url.replace('$basearch', repo.arch.name)
         if hasattr(settings, 'MAX_MIRRORS') and \
                 isinstance(settings.MAX_MIRRORS, int):
             max_mirrors = settings.MAX_MIRRORS
             # only add X mirrors, where X = max_mirrors
             q = Q(mirrorlist=False, refresh=True)
-            existing = mirror.repo.mirror_set.filter(q).count()
+            existing = repo.mirror_set.filter(q).count()
             if existing >= max_mirrors:
                 text = '{0!s} mirrors already '.format(max_mirrors)
                 text += 'exist, not adding {0!s}'.format(mirror_url)
                 warning_message.send(sender=None, text=text)
                 continue
         from repos.models import Mirror
-        m, c = Mirror.objects.get_or_create(repo=mirror.repo, url=mirror_url)
+        m, c = Mirror.objects.get_or_create(repo=repo, url=mirror_url)
         if c:
             text = 'Added mirror - {0!s}'.format(mirror_url)
             info_message.send(sender=None, text=text)
@@ -264,7 +251,7 @@ def check_for_mirrorlists(repo):
             mirror.save()
             text = 'Found mirrorlist - {0!s}'.format(mirror.url)
             info_message.send(sender=None, text=text)
-            add_mirrors_from_urls(mirror, mirror_urls)
+            add_mirrors_from_urls(repo, mirror_urls)
 
 
 def check_for_metalinks(repo):
@@ -282,7 +269,7 @@ def check_for_metalinks(repo):
             mirror.save()
             text = 'Found metalink - {0!s}'.format(mirror.url)
             info_message.send(sender=None, text=text)
-            add_mirrors_from_urls(mirror, mirror_urls)
+            add_mirrors_from_urls(repo, mirror_urls)
 
 
 def extract_yum_packages(data, url):
@@ -311,7 +298,6 @@ def extract_yum_packages(data, url):
                               namespaces={'ns': ns})[0].text
             fullversion = elem.xpath('//ns:version',
                                      namespaces={'ns': ns})[0]
-            epoch = fullversion.get('epoch')
             version = fullversion.get('ver')
             release = fullversion.get('rel')
             elem.clear()
@@ -319,10 +305,8 @@ def extract_yum_packages(data, url):
                 del elem.getparent()[0]
 
             if name != '' and version != '' and arch != '':
-                if epoch == '0':
-                    epoch = ''
                 package = PackageString(name=name,
-                                        epoch=epoch,
+                                        epoch='',
                                         version=version,
                                         release=release,
                                         arch=arch,
@@ -456,7 +440,6 @@ def refresh_yum_repo(mirror, data, mirror_url, ts):
     """ Refresh package metadata for a yum-style rpm mirror
         and add the packages to the mirror
     """
-
     primary_url, checksum, checksum_type = get_primary_url(mirror_url, data)
 
     if not primary_url:
@@ -475,13 +458,8 @@ def refresh_yum_repo(mirror, data, mirror_url, ts):
         mirror.fail()
         return
 
-    sha = get_sha(checksum_type, data)
-    if sha is None:
-        mirror.fail()
-        return
-
-    if not checksum_is_valid(sha, checksum, mirror):
-        mirror.fail()
+    computed_checksum = get_checksum(data, Checksum[checksum_type])
+    if not mirror_checksum_is_valid(computed_checksum, checksum, mirror):
         return
 
     if mirror.file_checksum == checksum:
@@ -509,27 +487,28 @@ def refresh_yum_repo(mirror, data, mirror_url, ts):
                 update_mirror_packages(mirror, packages)
 
 
-def checksum_is_valid(sha, checksum, mirror):
+def mirror_checksum_is_valid(computed, provided, mirror):
     """ Compares the computed checksum and the provided checksum. Returns True
         if both match.
     """
-
-    if sha == checksum:
-        return True
-    else:
+    if not computed or computed != provided:
         text = 'Checksum failed for mirror {0!s}'.format(mirror.id)
         text += ', not refreshing package metadata'
         error_message.send(sender=None, text=text)
-        text = 'Found sha = {0!s}\nExpected  = {1!s}'.format(sha, checksum)
+        text = 'Found checksum:    {0!s}\nExpected checksum: {1!s}'.format(computed, provided)
         error_message.send(sender=None, text=text)
         mirror.last_access_ok = False
+        mirror.fail()
         return False
+    else:
+        return True
 
 
 def refresh_arch_repo(repo):
     """ Refresh all mirrors of an arch linux repo
     """
     fname = '{0!s}.db'.format(repo.repo_id)
+    ts = datetime.now().replace(microsecond=0)
     for mirror in repo.mirror_set.filter(refresh=True):
         res = find_mirror_url(mirror.url, [fname])
         mirror.last_access_ok = response_is_valid(res)
@@ -542,17 +521,17 @@ def refresh_arch_repo(repo):
             if data is None:
                 mirror.fail()
                 return
-            sha1 = get_sha1(data)
-            if mirror.file_checksum == sha1:
+            computed_checksum = get_checksum(data, Checksum.sha1)
+            if mirror.file_checksum == computed_checksum:
                 text = 'Mirror checksum has not changed, '
                 text += 'not refreshing package metadata'
                 warning_message.send(sender=None, text=text)
             else:
                 packages = extract_arch_packages(data)
                 mirror.last_access_ok = True
-                mirror.timestamp = datetime.now()
+                mirror.timestamp = ts
                 update_mirror_packages(mirror, packages)
-                mirror.file_checksum = sha1
+                mirror.file_checksum = computed_checksum
                 packages.clear()
         else:
             mirror.fail()
@@ -604,14 +583,12 @@ def refresh_rpm_repo(repo):
 
     check_for_mirrorlists(repo)
     check_for_metalinks(repo)
-
     ts = datetime.now().replace(microsecond=0)
+    enabled_mirrors = repo.mirror_set.filter(mirrorlist=False, refresh=True)
 
-    for mirror in repo.mirror_set.filter(mirrorlist=False, refresh=True):
-
+    for mirror in enabled_mirrors:
         res = find_mirror_url(mirror.url, formats)
         mirror.last_access_ok = response_is_valid(res)
-
         if mirror.last_access_ok:
             data = download_url(res, 'Downloading repo info (1/2):')
             if data is None:
@@ -642,6 +619,7 @@ def refresh_deb_repo(repo):
     if lzma is not None:
         formats.insert(0, 'Packages.xz')
 
+    ts = datetime.now().replace(microsecond=0)
     for mirror in repo.mirror_set.filter(refresh=True):
         res = find_mirror_url(mirror.url, formats)
         mirror.last_access_ok = response_is_valid(res)
@@ -654,17 +632,17 @@ def refresh_deb_repo(repo):
             if data is None:
                 mirror.fail()
                 return
-            sha1 = get_sha1(data)
-            if mirror.file_checksum == sha1:
+            computed_checksum = get_checksum(data, Checksum.sha1)
+            if mirror.file_checksum == computed_checksum:
                 text = 'Mirror checksum has not changed, '
                 text += 'not refreshing package metadata'
                 warning_message.send(sender=None, text=text)
             else:
                 packages = extract_deb_packages(data, mirror_url)
                 mirror.last_access_ok = True
-                mirror.timestamp = datetime.now()
+                mirror.timestamp = ts
                 update_mirror_packages(mirror, packages)
-                mirror.file_checksum = sha1
+                mirror.file_checksum = computed_checksum
                 packages.clear()
         else:
             mirror.fail()
